@@ -1,12 +1,11 @@
-from redis import Redis, RedisError, ConnectionPool
+from redis import Redis, ConnectionPool
 import itertools
 import time
 import six
-from six.moves import zip
 
 from .document import Document
 from .result import Result
-from .query import Query, Filter
+from .query import Query
 from ._util import to_string
 from .aggregation import AggregateRequest, AggregateResult, Cursor
 
@@ -26,7 +25,7 @@ class Field(object):
     def __init__(self, name, *args):
         self.name = name
         self.args = args
-    
+
     def redis_args(self):
         return [self.name] + list(self.args)
 
@@ -87,13 +86,66 @@ class TagField(Field):
     TagField is a tag-indexing field with simpler compression and tokenization. 
     See http://redisearch.io/Tags/
     """
-    def __init__(self, name, separator = ',', no_index=False):
+
+    def __init__(self, name, separator=',', no_index=False):
         args = [Field.TAG, Field.SEPARATOR, separator]
 
         if no_index:
             args.append(Field.NOINDEX)
 
         Field.__init__(self, name, *args)
+
+
+class IndexDefinition(object):
+    """
+    IndexDefinition is used to define a index definition for automatic indexing on Hash update
+    """
+
+    ON = 'ON'
+    HASH = 'HASH'
+    PREFIX = 'PREFIX'
+    FILTER = 'FILTER'
+    LANGUAGE_FIELD = 'LANGUAGE_FIELD'
+    LANGUAGE = 'LANGUAGE'
+    SCORE_FIELD = 'SCORE_FIELD'
+    SCORE = 'SCORE'
+    PAYLOAD_FIELD = 'PAYLOAD_FIELD'
+        
+    def __init__(self, prefix=[], filter=None, language_field=None, language=None, score_field=None, score=1.0, payload_field=None):
+
+        args = [self.ON, self.HASH]
+
+        if len(prefix) > 0:
+            args.append(self.PREFIX)
+            args.append(len(prefix))
+            for p in prefix:
+                args.append(p)
+
+        if filter is not None:
+            args.append(self.FILTER)
+            args.append(filter)
+
+        if language_field is not None:
+            args.append(self.LANGUAGE_FIELD)
+            args.append(language_field)
+
+        if language is not None:
+            args.append(self.LANGUAGE)
+            args.append(language)
+
+        if score_field is not None:
+            args.append(self.SCORE_FIELD)
+            args.append(score_field)
+
+        if score is not None:
+            args.append(self.SCORE)
+            args.append(score)
+
+        if payload_field is not None:
+            args.append(self.PAYLOAD_FIELD)
+            args.append(payload_field)
+
+        self.args = args
     
 
 class Client(object):
@@ -108,6 +160,7 @@ class Client(object):
     ALTER_CMD = 'FT.ALTER'
     SEARCH_CMD = 'FT.SEARCH'
     ADD_CMD = 'FT.ADD'
+    ADDHASH_CMD = "FT.ADDHASH"
     DROP_CMD = 'FT.DROP'
     EXPLAIN_CMD = 'FT.EXPLAIN'
     DEL_CMD = 'FT.DEL'
@@ -119,7 +172,11 @@ class Client(object):
     DICT_DUMP_CMD = 'FT.DICTDUMP'
     GET_CMD = 'FT.GET'
     MGET_CMD = 'FT.MGET'
-
+    CONFIG_CMD = 'FT.CONFIG'
+    TAGVALS_CMD = 'FT.TAGVALS'
+    ALIAS_ADD_CMD = 'FT.ALIASADD'
+    ALIAS_UPDATE_CMD = 'FT.ALIASUPDATE'
+    ALIAS_DEL_CMD = 'FT.ALIASDEL'
 
     NOOFFSETS = 'NOOFFSETS'
     NOFIELDS = 'NOFIELDS'
@@ -156,6 +213,20 @@ class Client(object):
             if self.current_chunk >= self.chunk_size:
                 self.commit()
 
+        def add_document_hash(
+            self, doc_id, score=1.0, replace=False,
+        ):
+            """
+            Add a hash to the batch query
+            """
+            self.client._add_document_hash(
+                doc_id, conn=self.pipeline, score=score, replace=replace,
+            )
+            self.current_chunk += 1
+            self.total += 1
+            if self.current_chunk >= self.chunk_size:
+                self.commit()
+
         def commit(self):
             """
             Manually commit and flush the batch indexing query
@@ -173,7 +244,8 @@ class Client(object):
         self.index_name = index_name
 
         self.redis = conn if conn is not None else Redis(
-            connection_pool=ConnectionPool(host=host, port=port, password=password))
+            connection_pool=ConnectionPool(host=host, port=port, password=password,
+            decode_responses=True))
 
     def batch_indexer(self, chunk_size=100):
         """
@@ -182,7 +254,7 @@ class Client(object):
         return Client.BatchIndexer(self, chunk_size=chunk_size)
 
     def create_index(self, fields, no_term_offsets=False,
-                     no_field_flags=False, stopwords = None):
+                     no_field_flags=False, stopwords=None, definition=None):
         """
         Create the search index. The index must not already exist.
 
@@ -195,6 +267,8 @@ class Client(object):
         """
 
         args = [self.CREATE_CMD, self.index_name]
+        if definition is not None:
+            args += definition.args            
         if no_term_offsets:
             args.append(self.NOOFFSETS)
         if no_field_flags:
@@ -203,7 +277,7 @@ class Client(object):
             args += [self.STOPWORDS, len(stopwords)]
             if len(stopwords) > 0:
                 args += list(stopwords)
-    
+
         args.append('SCHEMA')
 
         args += list(itertools.chain(*(f.redis_args() for f in fields)))
@@ -230,7 +304,7 @@ class Client(object):
         Drop the index if it exists
         """
         return self.redis.execute_command(self.DROP_CMD, self.index_name)
-        
+
     def _add_document(self, doc_id, conn=None, nosave=False, score=1.0, payload=None,
                       replace=False, partial=False, language=None, no_create=False, **fields):
         """ 
@@ -260,6 +334,25 @@ class Client(object):
         args += list(itertools.chain(*fields.items()))
         return conn.execute_command(*args)
 
+    def _add_document_hash(
+        self, doc_id, conn=None, score=1.0, language=None, replace=False,
+    ):
+        """ 
+        Internal add_document_hash used for both batch and single doc indexing 
+        """
+        if conn is None:
+            conn = self.redis
+
+        args = [self.ADDHASH_CMD, self.index_name, doc_id, score]
+
+        if replace:
+            args.append("REPLACE")
+
+        if language:
+            args += ["LANGUAGE", language]
+
+        return conn.execute_command(*args)
+
     def add_document(self, doc_id, nosave=False, score=1.0, payload=None,
                      replace=False, partial=False, language=None, no_create=False, **fields):
         """
@@ -281,20 +374,44 @@ class Client(object):
         - **fields** kwargs dictionary of the document fields to be saved and/or indexed. 
                      NOTE: Geo points shoule be encoded as strings of "lon,lat"
         """
-        return self._add_document(doc_id, conn=None, nosave=nosave, score=score, 
+        return self._add_document(doc_id, conn=None, nosave=nosave, score=score,
                                   payload=payload, replace=replace,
-                                  partial=partial, language=language, 
-                                  no_create=no_create,**fields)
+                                  partial=partial, language=language,
+                                  no_create=no_create, **fields)
 
-    def delete_document(self, doc_id, conn=None):
+    def add_document_hash(
+        self, doc_id, score=1.0, language=None, replace=False,
+    ):
+        """
+        Add a hash document to the index.
+
+        ### Parameters
+
+        - **doc_id**: the document's id. This has to be an existing HASH key in Redis that will hold the fields the index needs.
+        - **score**: the document ranking, between 0.0 and 1.0 
+        - **replace**: if True, and the document already is in the index, we perform an update and reindex the document
+        - **language**: Specify the language used for document tokenization.
+        """
+        return self._add_document_hash(
+            doc_id, conn=None, score=score, language=language, replace=replace,
+        )
+
+    def delete_document(self, doc_id, conn=None, delete_actual_document=False):
         """
         Delete a document from index
         Returns 1 if the document was deleted, 0 if not
+
+        ### Parameters
+
+        - **delete_actual_document**: if set to True, RediSearch also delete the actual document if it is in the index
         """
+        args = [self.DEL_CMD, self.index_name, doc_id]
         if conn is None:
             conn = self.redis
+        if delete_actual_document:
+            args.append('DD')
 
-        return conn.execute_command(self.DEL_CMD, self.index_name, doc_id)
+        return conn.execute_command(*args)
 
     def load_document(self, id):
         """
@@ -315,12 +432,12 @@ class Client(object):
     def get(self, *ids):
         """
         Returns the full contents of multiple documents.
-         
+
         ### Parameters
-        
+
         - **ids**: the ids of the saved documents.
         """
-        
+
         return self.redis.execute_command('FT.MGET', self.index_name, *ids)
 
     def info(self):
@@ -352,7 +469,6 @@ class Client(object):
 
         - **query**: the search query. Either a text for simple queries with default parameters, or a Query object for complex queries.
                      See RediSearch's documentation on query format
-        - **snippet_sizes**: A dictionary of {field: snippet_size} used to trim and format the result. e.g.e {'body': 500}
         """
         args, query = self._mk_query_args(query)
         st = time.time()
@@ -380,13 +496,12 @@ class Client(object):
         `rows` property, which will always yield the rows of the result
         """
         if isinstance(query, AggregateRequest):
-            has_schema = query._with_schema
             has_cursor = bool(query._cursor)
             cmd = [self.AGGREGATE_CMD, self.index_name] + query.build_args()
         elif isinstance(query, Cursor):
-            has_schema = False
             has_cursor = True
-            cmd = [self.CURSOR_CMD, 'READ', self.index_name] + query.build_args()
+            cmd = [self.CURSOR_CMD, 'READ',
+                   self.index_name] + query.build_args()
         else:
             raise ValueError('Bad query', query)
 
@@ -401,7 +516,7 @@ class Client(object):
         else:
             cursor = None
 
-        if query._with_schema:
+        if isinstance(query, AggregateRequest) and query._with_schema:
             schema = raw[0]
             rows = raw[2:]
         else:
@@ -439,7 +554,7 @@ class Client(object):
             return corrections
 
         for _correction in raw:
-            if isinstance(_correction, long) and _correction == 0:
+            if isinstance(_correction, six.integer_types) and _correction == 0:
                 continue
 
             if len(_correction) != 3:
@@ -507,3 +622,78 @@ class Client(object):
         cmd = [self.DICT_DUMP_CMD, name]
         raw = self.redis.execute_command(*cmd)
         return raw
+
+    def config_set(self, option, value):
+        """Set runtime configuration option.
+
+        ### Parameters
+
+        - **option**: the name of the configuration option.
+        - **value**: a value for the configuration option.
+        """
+        cmd = [self.CONFIG_CMD, 'SET', option, value]
+        raw = self.redis.execute_command(*cmd)
+        return raw == 'OK'
+
+    def config_get(self, option):
+        """Get runtime configuration option value.
+
+        ### Parameters
+
+        - **option**: the name of the configuration option.
+        """
+        cmd = [self.CONFIG_CMD, 'GET', option]
+        res = {}
+        raw = self.redis.execute_command(*cmd)
+        if raw:
+            for kvs in raw:
+                res[kvs[0]] = kvs[1]
+        return res
+
+    def tagvals(self, tagfield):
+        """
+        Return a list of all possible tag values
+
+        ### Parameters
+
+        - **tagfield**: Tag field name
+        """
+
+        cmd = self.redis.execute_command(self.TAGVALS_CMD, self.index_name, tagfield)
+        return cmd
+
+    def aliasadd(self, alias):
+        """
+        Alias a search index - will fail if alias already exists
+
+        ### Parameters
+
+        - **alias**: Name of the alias to create
+        """
+
+        cmd = self.redis.execute_command(self.ALIAS_ADD_CMD, alias, self.index_name)
+        return cmd
+
+    def aliasupdate(self, alias):
+        """
+        Updates an alias - will fail if alias does not already exist
+
+        ### Parameters
+
+        - **alias**: Name of the alias to create
+        """
+
+        cmd = self.redis.execute_command(self.ALIAS_UPDATE_CMD, alias, self.index_name)
+        return cmd
+
+    def aliasdel(self, alias):
+        """
+        Removes an alias to a search index
+
+        ### Parameters
+
+        - **alias**: Name of the alias to delete
+        """
+
+        cmd = self.redis.execute_command(self.ALIAS_DEL_CMD, alias)
+        return cmd
\ No newline at end of file
